﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// メインルーチンクラスです。
    /// </summary>
    class Program
    {
        /// <summary>
        /// ループ処理。
        /// </summary>
        /// <returns>エラーなら -1、正常なら 0</returns>
        private static int ProcessLoop()
        {
            if (DX.ProcessMessage() != 0)
                return -1; // プロセス処理がエラーなら -1を返す
            if (DX.ClearDrawScreen() != 0)
                return -1; // 画面クリア処理がエラーなら -1を返す

            KeyManager keyManager = KeyManager.GetInstance();
            keyManager.GetHitKeyStateAll_2(); //現在のキー入力処理を行う
            keyManager.GetHitPadStateAll(); //現在のパッド入力処理を行う

            return 0;
        }

        /// <summary>
        /// メインメソッド
        /// </summary>
        /// <param name="args">引数</param>
        public static void Main(string[] args)
        {
            Debug.Listeners.Add(new TextWriterTraceListener(Console.Out));
            Debug.WriteLine("起動しました。");

            DX.ChangeWindowMode(1); // ウインドウモード

            // 初期化と裏画面化
            if (DX.DxLib_Init() == -1 || DX.SetDrawScreen(DX.DX_SCREEN_BACK) != 0)
                return;

            while (ProcessLoop() == 0)
            {
                switch (GV.func_state)
                {
                    case 0:
                        // データロード
                        LoadManager loadManager = LoadManager.GetInstance();
                        loadManager.load();
                        GV.first_ini(); // 初回の初期化
                        GV.func_state = 100;
                        break;
                    case 100:
                        GV.ch.calc_ch(); // キャラクタ計算

                        GraphManager graphManager = GraphManager.GetInstance();
                        graphManager.graph_main(); // 描画メイン
                        break;
                    default:
                        Debug.WriteLine("不明な func_state");
                        break;
                }
                
                KeyManager keyManager = KeyManager.GetInstance();
                if (keyManager.CheckStateKey(DX.KEY_INPUT_ESCAPE) == 1)
                    break; //エスケープが入力されたらブレイク

                DX.ScreenFlip(); // 裏画面反映
            }

            DX.DxLib_End(); // DXライブラリ終了処理
        }
    }
}
