﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    class EnemyGroup
    {
        /// <summary>
        /// 敵群。
        /// </summary>
        private Enemy[] enemy = new Enemy[Setting.ENEMY_MAX];

        /// <summary>
        /// 敵にアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>敵</returns>
        public Enemy this[int index]
        {
            get
            {
                return this.enemy[index];
            }

            set
            {
                this.enemy[index] = value;
            }
        }

        /// <summary>
        /// 敵の数を返します。
        /// </summary>
        /// <returns>敵の数</returns>
        public int length()
        {
            return this.enemy.Length;
        }

        /// <summary>
        /// ゲームの初期化をします。
        /// </summary>
        public void ini()
        {
            for (int i = 0; i < this.enemy.Length; i++)
            {
                this.enemy[i] = new Enemy();
            }
        }

        /// <summary>
        /// 敵データを登録します。
        /// </summary>
        private void enemy_enter()
        {
            if (GV.stage_count == 100) // ゲームが始まって 100カウントで登録
            {
                this.enemy[0].cnt = 0;
                this.enemy[0].muki = 1;
                this.enemy[0].flag = 1;
                this.enemy[0].bltime = 150;
                this.enemy[0].hp = 1000;
                this.enemy[0].hp_max = this.enemy[0].hp;
                this.enemy[0].pattern = 0;
                this.enemy[0].x = Setting.FIELD_MAX_X / 2.0;
                this.enemy[0].y = -20.0;
            }
        }

        /// <summary>
        /// 敵の行動制御をします。
        /// </summary>
        private void enemy_act()
        {
            for (int i = 0; i < this.enemy.Length; i++)
            {
                Enemy cEnemy = this.enemy[i];
                if(cEnemy.flag == 1) // その敵のフラグがオンになっていたら
                {
                    cEnemy.enemy_pattern0();
                    cEnemy.cnt++;
                    cEnemy.img = cEnemy.muki * 3 + (cEnemy.cnt % 18) / 6;

                    // 敵が画面から外れたら消す
                    if (cEnemy.x < 50 || Setting.FIELD_MAX_X + 50 < cEnemy.x || cEnemy.y < -50 || Setting.FIELD_MAX_Y + 50 < cEnemy.y)
                        cEnemy.flag = 0;
                }
            }
        }

        /// <summary>
        /// 敵処理メイン
        /// </summary>
        public void enemy_main()
        {
            enemy_enter();
            enemy_act();
        }
    }
}
