﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 敵の弾幕を表すクラスです。
    /// </summary>
    class Shot
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// どの敵から発射されたか。
        /// </summary>
        public Enemy parent_enemy;

        /// <summary>
        /// ベース角度。
        /// </summary>
        public double[] base_angle = new double[1];

        /// <summary>
        /// ベーススピード。
        /// </summary>
        public double[] base_spd = new double[1];

        /// <summary>
        /// 弾群。
        /// </summary>
        public BulletGroup bullets = new BulletGroup();

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.knd = 0;
            this.cnt = 0;
            this.parent_enemy = null;
            for (int i = 0; i < this.base_angle.Length; i++)
                this.base_angle[i] = 0;
            for (int i = 0; i < this.base_spd.Length; i++)
                this.base_spd[i] = 0;
            this.bullets.initialize();
        }
        
        /// <summary>
        /// 弾を発射した敵と自キャラクタとの角度を返します。
        /// </summary>
        /// <returns>角度</returns>
        private double shotatan2()
        {
            return Math.Atan2(GV.ch.y - this.parent_enemy.y, GV.ch.x - this.parent_enemy.x);
        }

        /// <summary>
        /// 1発だけ自キャラクタに向かって直線移動。
        /// </summary>
        private void shot_bullet_H000()
        {
            if (this.cnt == 0)
            {
                if (this.flag != 2)
                {
                    Bullet bullet = this.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = this.parent_enemy.blknd2;
                        bullet.angle = this.shotatan2();
                        bullet.flag = 1;
                        bullet.x = this.parent_enemy.x;
                        bullet.y = this.parent_enemy.y;
                        bullet.col = this.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }
        
        /// <summary>
        /// 100カウント中に 10発、自キャラクタに向かって直線発射(常に自キャラクタ狙い)
        /// </summary>
        private void shot_bullet_H001()
        {
            if(this.cnt >= 0 && this.cnt < 100 && this.cnt % 10 == 0) // 100カウント中 10カウントに 1回
                if (this.flag != 2) // 敵がいて
                {
                    Bullet bullet = this.bullets.search_unused_bullet();
                    if (bullet != null) // 弾が登録可能なら
                    {
                        bullet.knd = this.parent_enemy.blknd2; // 弾登録
                        bullet.angle = this.shotatan2(); // 角度
                        bullet.flag = 1; // フラグ
                        bullet.x = this.parent_enemy.x; // X座標
                        bullet.y = this.parent_enemy.y; // Y座標
                        bullet.col = this.parent_enemy.col; // 色
                        bullet.cnt = 0; // カウンタ
                        bullet.spd = 3.0; // スピード

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
        }

        /// <summary>
        /// 100カウント中に 10発、自キャラクタに向かって直線発射(角度記憶)
        /// </summary>
        private void shot_bullet_H002()
        {
            if (this.cnt >= 0 && this.cnt < 100 && this.cnt % 10 == 0)
            {
                if (this.cnt == 0)
                    this.base_angle[0] = this.shotatan2();
                if (this.flag != 2)
                {
                    Bullet bullet = this.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = this.parent_enemy.blknd2;
                        bullet.angle = this.base_angle[0];
                        bullet.flag = 1;
                        bullet.x = this.parent_enemy.x;
                        bullet.y = this.parent_enemy.y;
                        bullet.col = this.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }
        
        /// <summary>
        /// 100カウント中に 10発、自キャラクタに向かってスピード変化直線発射。
        /// </summary>
        private void shot_bullet_H003()
        {
            if(this.cnt >= 0 && this.cnt < 100 && this.cnt % 10 == 0)
                if (this.flag != 2)
                {
                    Bullet bullet = this.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = this.parent_enemy.blknd2;
                        bullet.angle = shotatan2();
                        bullet.flag = 1;
                        bullet.x = this.parent_enemy.x;
                        bullet.y = this.parent_enemy.y;
                        bullet.col = this.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 1.0 + 5.0 / 100.0 * this.cnt;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
        }

        /// <summary>
        /// 0.5秒に 1回ずつ円形発射。
        /// </summary>
        private void shot_bullet_H004()
        {
            if (this.cnt >= 0 && this.cnt < 120 && this.cnt % 20 == 0)
            {
                double angle = this.shotatan2();
                for (int i = 0; i < 20; i++)
                {
                    if (this.flag != 2)
                    {
                        Bullet bullet = this.bullets.search_unused_bullet();
                        if (bullet != null)
                        {
                            bullet.knd = this.parent_enemy.blknd2;
                            bullet.angle = angle + Math.PI * 2.0 / 20.0 * i;
                            bullet.flag = 1;
                            bullet.x = this.parent_enemy.x;
                            bullet.y = this.parent_enemy.y;
                            bullet.col = this.parent_enemy.col;
                            bullet.cnt = 0;
                            bullet.spd = 4.0;

                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;
                        }
                    }
                }
            }
        }
        
        /// <summary>
        /// ばらまきショット。
        /// </summary>
        private void shot_bullet_H005()
        {
            if(this.cnt >= 0 && this.cnt < 120 && this.cnt % 2 == 0)
                if (this.flag != 2)
                {
                    Bullet bullet = this.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = this.parent_enemy.blknd2;
                        bullet.angle = this.shotatan2() + MyMath.rang(Math.PI / 4.0);
                        bullet.flag = 1;
                        bullet.x = this.parent_enemy.x;
                        bullet.y = this.parent_enemy.y;
                        bullet.col = this.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 3.0 + MyMath.rang(1.5);

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
        }
        
        /// <summary>
        /// ばらまきショット(減速)。
        /// </summary>
        private void shot_bullet_H006()
        {
            if(this.cnt >= 0 && this.cnt <= 120 && this.cnt % 2 == 0)
                if (this.flag != 2)
                {
                    Bullet bullet = this.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = this.parent_enemy.blknd2;
                        bullet.angle = this.shotatan2() + MyMath.rang(Math.PI / 4.0);
                        bullet.flag = 1;
                        bullet.x = this.parent_enemy.x;
                        bullet.y = this.parent_enemy.y;
                        bullet.col = this.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 4.0 + MyMath.rang(2.0);

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }

            for (int i = 0; i < this.bullets.length(); i++) // 全弾分ループ
            {
                Bullet bullet = this.bullets[i];
                if (bullet.spd > 1.5) // 速度が 1.5より大きいものがあれば
                    bullet.spd -= 0.04; // 減速
            }
        }

        /// <summary>
        /// 弾幕を処理します。
        /// </summary>
        public void do_shot_bullet()
        {
            switch (this.knd)
            {
                case 0:
                    this.shot_bullet_H000();
                    break;
                case 1:
                    this.shot_bullet_H001();
                    break;
                case 2:
                    this.shot_bullet_H002();
                    break;
                case 3:
                    this.shot_bullet_H003();
                    break;
                case 4:
                    this.shot_bullet_H004();
                    break;
                case 5:
                    this.shot_bullet_H005();
                    break;
                case 6:
                    this.shot_bullet_H006();
                    break;
                default:
                    Debug.WriteLine("弾幕の種類が異常値(" + this.knd + ")です");
                    break;
            }

        }
        
        /// <summary>
        /// 弾幕を計算します。
        /// </summary>
        public void calc()
        {
            if (this.parent_enemy.flag != 1) // 敵が倒されたら
                this.flag = 2; // それ以上弾を登録しないフラグに変える
            this.bullets.calc(); // 弾群について計算する

            if (this.bullets.exist_display_bullet()) // 表示中の弾がひとつでもあれば
                return; // 処理を終える

            // 表示中の弾がひとつも無い場合
            if (this.parent_enemy.flag != 1) // 敵が倒されたら
            {
                this.flag = 0; // 終了
                this.parent_enemy.flag = 0; // 処理不明
            }
        }
    }
}
