﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 弾幕群を管理するクラスです。
    /// </summary>
    class ShotGroup
    {
        /// <summary>
        /// 弾幕群。
        /// </summary>
        private Shot[] shots = new Shot[Setting.SHOT_MAX];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public ShotGroup()
        {
            for (int i = 0; i < this.shots.Length; i++)
            {
                this.shots[i] = new Shot();
            }
        }

        /// <summary>
        /// 弾幕にアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>弾幕</returns>
        public Shot this[int index]
        {
            get
            {
                return this.shots[index];
            }
        }

        /// <summary>
        /// 弾幕の数を返します。
        /// </summary>
        /// <returns>弾幕の数</returns>
        public int length()
        {
            return this.shots.Length;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.shots.Length; i++)
                this.shots[i].initialize();
        }

        /// <summary>
        /// 未使用の弾幕を検索して返します。
        /// </summary>
        /// <returns>未使用の弾幕。無い場合には null</returns>
        public Shot search_unused_shot()
        {
            for (int i = 0; i < this.shots.Length; i++)
            {
                Shot shot = this.shots[i];
                if (shot.flag == 0)
                    return shot; // 未使用の弾幕を返す
            }

            return null; // 未使用の弾幕が無い場合には nullを返す
        }

        /// <summary>
        /// 弾幕を処理します。
        /// </summary>
        public void shot_main()
        {
            for (int i = 0; i < this.shots.Length; i++)
            {
                Shot shot = this.shots[i];
                if (shot.flag != 0)
                {
                    // ●
                    shot.do_shot_bullet(); // 弾幕計算関数を実行
                    // ●
                    shot.calc(); // 弾幕を計算
                    shot.cnt++;
                }
            }
        }
    }
}
