﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// エフェクト群を管理するクラスです。
    /// </summary>
    class EffectGroup
    {
        /// <summary>
        /// エフェクト群。
        /// </summary>
        private Effect[] effects = new Effect[Setting.EFFECT_MAX];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public EffectGroup()
        {
            for (int i = 0; i < this.effects.Length; i++)
            {
                this.effects[i] = new Effect();
            }
        }

        /// <summary>
        /// エフェクトにアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>エフェクト</returns>
        public Effect this[int index]
        {
            get
            {
                return this.effects[index];
            }
        }

        /// <summary>
        /// エフェクトの数を返します。
        /// </summary>
        /// <returns>エフェクトの数</returns>
        public int length()
        {
            return this.effects.Length;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.effects.Length; i++)
                this.effects[i].initialize();
        }
        
        /// <summary>
        /// エフェクトの登録空き番号を返します。
        /// </summary>
        /// <returns>エフェクト。無い場合は null</returns>
        public Effect search_effect()
        {
            for (int i = 0; i < this.effects.Length; i++)
            {
                Effect effect = this.effects[i];
                if (effect.flag == 0)
                    return effect;
            }

            return null;
        }
        
        /// <summary>
        /// エフェクトを計算します。
        /// </summary>
        public void calc_effect()
        {
            for (int i = 0; i < this.effects.Length; i++)
            {
                Effect effect = this.effects[i];
                if (effect.flag > 0) // エフェクトが登録されていたら
                    effect.calc_effect();
            }
        }
        
        /// <summary>
        /// エフェクトのメイン処理です。
        /// </summary>
        public void effect_main()
        {
            GV.del_effects.calc_del_effect(); // 消滅エフェクトの計算
            this.calc_effect(); // エフェクトの計算
        }
    }
}
