﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボムを表すクラスです。
    /// </summary>
    class Bom
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.knd = 0;
            this.x = 0.0;
            this.y = 0.0;
        }

        /// <summary>
        /// 敵に power食らわします。
        /// </summary>
        /// <param name="enemy">敵</param>
        /// <param name="power">パワー</param>
        private void hit_enemy(Enemy enemy, int power)
        {
            enemy.decrease_hp(power);
            enemy.enemy_death_judge();
        }

        /// <summary>
        /// ボムと敵の処理をします。
        /// </summary>
        public void cbom_and_enemy()
        {
            if (this.flag != 1)
                return;
            for (int s = 0; s < GV.enemys.length(); s++)
            {
                Enemy enemy = GV.enemys[s];
                if (enemy.flag > 0)
                    this.hit_enemy(enemy, GV.ch.power / 20); // 敵に ch.power/20ほど食らわす
            }
        }
        
        /// <summary>
        /// ボムを登録します。
        /// </summary>
        private void enter_bom()
        {
            Effect effect;
            MusicManager musicManager = MusicManager.GetInstance();
            ImageManager imageManager = ImageManager.GetInstance();

            this.flag = 1;
            this.cnt = 0;
            this.knd = 0;
            GV.ch.setMuteki();
            musicManager.se_flag[14] = 1; // キュイーン音

            // 縦線
            effect = GV.effects.search_effect();
            if (effect != null)
            {
                effect.flag = 1;
                effect.cnt = 0;
                effect.knd = 3; // ボムの線のエフェクト
                effect.brt = 0.0; // α値
                effect.ang = Math.PI / 2; // 画像の向き
                effect.mvang = -Math.PI / 2; // エフェクトが進む方向
                effect.spd = 1.0; // スピード
                effect.r = 1.0; // 大きさ
                effect.eff = 2; // αブレンド
                effect.img = imageManager.img_eff_bom[3];
                effect.x = 70.0; // X座標
                effect.y = 300.0; // Y座標
            }

            // 横線
            effect = GV.effects.search_effect();
            if (effect != null)
            {
                effect.flag = 1;
                effect.cnt = 0;
                effect.knd = 3; // ボムの線のエフェクト
                effect.brt = 0.0; // α値
                effect.ang = 0.0; // 画像の向き
                effect.mvang = 0.0; // エフェクトが進む方向
                effect.spd = 1.0; // スピード
                effect.r = 1.0; // 大きさ
                effect.eff = 2; // αブレンド
                effect.img = imageManager.img_eff_bom[3];
                effect.x = 100.0; // X座標
                effect.y = 350.0; // Y座標
            }

            // キャラ
            effect = GV.effects.search_effect();
            if (effect != null)
            {
                effect.flag = 1;
                effect.cnt = 0;
                effect.knd = 2; // ボムのキャラのエフェクト
                effect.brt = 0.0; // α値
                effect.ang = 0.0; // 画像の向き
                effect.mvang = - Math.PI / 2; // エフェクトが進む方向
                effect.spd = 0.7; // スピード
                effect.r = 1.0; // 大きさ
                effect.eff = 2; // αブレンド
                effect.img = imageManager.img_eff_bom[2];
                effect.x = 260.0; // X座標
                effect.y = 300.0; // Y座標
            }
        }
        
        /// <summary>
        /// 喰らいボムのエフェクトを登録します。
        /// ボムとは関係ないのだけど、このクラスに記述します。
        /// </summary>
        public void enter_crybom_effect()
        {
            if(GV.ch.flag == 1)
                if (GV.ch.cnt % 2 == 0)
                {
                    Effect effect = GV.effects.search_effect();
                    if (effect != null)
                    {
                        ImageManager imageManager = ImageManager.GetInstance();
                        effect.flag = 1;
                        effect.cnt = 0;
                        effect.knd = 4; // 喰らいボムエフェクト
                        effect.brt = 255.0;
                        effect.ang = MyMath.rang(Math.PI);
                        effect.spd = 0.0;
                        effect.r = 0;
                        effect.eff = 2;
                        effect.img = imageManager.img_del_effect[MyMath.GetRand(4)];
                        effect.x = GV.ch.x;
                        effect.y = GV.ch.y;
                    }
                }
        }

        /// <summary>
        /// ボムの計算をします。
        /// </summary>
        public void bom_calc()
        {
            double[] shot_angle = new double[] { 0, Math.PI, Math.PI / 2, Math.PI * 1.5 }; // 4発エフェクト

            if ((GV.ch.flag == 0 || GV.ch.flag == 1) && this.flag == 0) // キャラが通常状態か喰らいボム状態で、ボムがオフなら
            {
                KeyManager keyManager = KeyManager.GetInstance();
                if (keyManager.ChckStatePad(GV.configpad.bom) == 1) // ボムボタンが押されたら
                    this.enter_bom();
            }

            if (this.flag == 1) // ボムが登録されていたら
            {
                if (this.cnt % 10 == 0) // 1/6秒に 1回
                {
                    int n = this.cnt / 10;
                    if (n < 4)
                    {
                        Effect effect = GV.effects.search_effect();
                        if (effect != null)
                        {
                            ImageManager imageManager = ImageManager.GetInstance();
                            effect.flag = 1;
                            effect.cnt = 0;
                            effect.knd = 1; // エフェクト番号 1: ボムのエフェクト
                            effect.brt = 0.0;
                            effect.ang = MyMath.rang(Math.PI); // 画像の向きはランダム
                            effect.mvang = shot_angle[n] - Math.PI / 4; // 飛ぶ方向
                            effect.spd = 13.0 + MyMath.rang(2.0); // スピード
                            effect.r = 0.5; // 大きさ
                            effect.eff = 2; // αブレンド
                            effect.img = imageManager.img_eff_bom[(this.cnt / 10) / 3]; // 画像
                            effect.x = GV.ch.x; // X座標
                            effect.y = GV.ch.y; // Y座標
                        }
                    }
                }

                this.cnt++;
                if (this.cnt < 40)
                    GV.bright_set.setBright(255 - this.cnt * 5); // 画面の明るさ設定(暗く)
                if (this.cnt > 90)
                    GV.bright_set.setBright(255 - (40 * 5) + (this.cnt - 90) * 5); // 画面の明るさ設定(明るく)
                if (this.cnt > 130)
                {
                    this.flag = 0;
                    GV.bright_set.setBright(255);
                }
            }
        }
    }
}
