﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// グラフィックスを管理するクラスです。
    /// </summary>
    class GraphManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static GraphManager manager = new GraphManager();

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private GraphManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static GraphManager GetInstance()
        {
            return manager;
        }
        
        /// <summary>
        /// 背景番号 0の背景を描画します。
        /// </summary>
        private void graph_back_00()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawArea(Setting.FIELD_X, Setting.FIELD_Y, Setting.FIELD_X + Setting.FIELD_MAX_X, Setting.FIELD_Y + Setting.FIELD_MAX_Y); // 描画可能エリアを設定
            DX.DrawGraph(Setting.FIELD_X, GV.count % 700 + Setting.FIELD_Y - 700, imageManager.img_back[0], 0);
            DX.DrawGraph(Setting.FIELD_X, GV.count % 700 + Setting.FIELD_Y, imageManager.img_back[0], 0);
            DX.SetDrawArea(0, 0, 640, 480); // エリアを戻す
        }

        /// <summary>
        /// 背景を描画するメイン処理です。
        /// </summary>
        private void graph_back_main()
        {
            this.graph_back_00();
        }

        /// <summary>
        /// エフェクト群を描画します。
        /// </summary>
        /// <param name="knd">エフェクトの種類</param>
        private void graph_effect(int knd)
        {
            for (int i = 0; i < GV.effects.length(); i++)
            {
                Effect effect = GV.effects[i];
                if (effect.flag > 0 && effect.knd == knd)
                {
                    if (effect.eff == 1) // エフェクトが光エフェクトなら
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, (int)effect.brt);
                    if (effect.eff == 2) // エフェクトがαエフェクトなら
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_ALPHA, (int)effect.brt);

                    DX.DrawRotaGraphF((float)effect.x + Setting.FIELD_X + GV.dn.x, (float)effect.y + Setting.FIELD_Y + GV.dn.y,
                        effect.r, effect.ang, effect.img, 1);

                    if (effect.eff == 1 || effect.eff == 2)
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                }
            }
        }

        /// <summary>
        /// ボスを描画します。
        /// </summary>
        private void graph_boss()
        {
            if (GV.boss.flag == 0)
                return;
            ImageManager imageManager = ImageManager.GetInstance();
            DX.DrawRotaGraphF((float)GV.boss.x + Setting.FIELD_X + GV.dn.x, (float)GV.boss.y + Setting.FIELD_Y + GV.dn.y, 1.0, 0.0, imageManager.img_dot_riria[0], 1);
        }

        /// <summary>
        /// 敵キャラクタ群を描画します。
        /// </summary>
        private void graph_enemy()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.enemys.length(); i++)
            {
                Enemy enemy = GV.enemys[i];
                if (enemy.flag == 1)
                    DX.DrawRotaGraphF((float)enemy.x + Setting.FIELD_X + GV.dn.x, (float)enemy.y + Setting.FIELD_Y + GV.dn.y, 1.0, 0.0, imageManager.img_enemy[0, enemy.img], 1);
            }
        }

        /// <summary>
        /// ショットを描画します。
        /// </summary>
        private void graph_cshot()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.cshots.length(); i++)
            {
                CShot cshot = GV.cshots[i];
                if (cshot.flag > 0)
                    DX.DrawRotaGraphF((float)cshot.x + Setting.FIELD_X + GV.dn.x, (float)cshot.y + Setting.FIELD_Y + GV.dn.y, 1.0, 0.0, imageManager.img_cshot[cshot.knd], 1);
            }
        }

        /// <summary>
        /// 自キャラクタを描画します。
        /// </summary>
        private void graph_ch()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            if(GV.ch.mutekicnt % 2 == 0)
                DX.DrawRotaGraphF((float)GV.ch.x + Setting.FIELD_X + GV.dn.x, (float)GV.ch.y + Setting.FIELD_Y + GV.dn.y, 1.0, 0.0, imageManager.img_ch[0, GV.ch.img], 1);
        }

        /// <summary>
        /// 弾幕を描画します。
        /// </summary>
        private void graph_bullet()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawMode(DX.DX_DRAWMODE_BILINEAR); // 線形補間描画

            // 敵弾
            for (int i = 0; i < GV.shots.length(); i++) // 弾幕数分ループ
            {
                Shot shot = GV.shots[i];
                if (shot.flag > 0) // 弾幕データがオンなら
                {
                    BulletGroup bullets = shot.bullets;
                    for (int j = 0; j < bullets.length(); j++) // 弾幕が持つの弾の数分ループ
                    {
                        Bullet bullet = bullets[j];
                        if (bullet.flag != 0) // 弾データがオンなら
                        {
                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);

                            DX.DrawRotaGraphF((float)bullet.x + Setting.FIELD_X + GV.dn.x, (float)bullet.y + Setting.FIELD_Y + GV.dn.y,
                                1.0, bullet.angle + Math.PI / 2.0,
                                imageManager.img_bullet[bullet.knd, bullet.col], 1);

                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                        }
                    }
                }
            }

            // ボス弾
            if(GV.boss_shot.flag > 0) // 弾幕データがオンなら
            {
                BulletGroup bullets = GV.boss_shot.bullets;
                for (int j = 0; j < bullets.length(); j++) // 弾幕が持つの弾の数分ループ
                {
                    Bullet bullet = bullets[j];
                    if (bullet.flag != 0) // 弾データがオンなら
                    {
                        if (bullet.eff == 1)
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);

                        DX.DrawRotaGraphF((float)bullet.x + Setting.FIELD_X + GV.dn.x, (float)bullet.y + Setting.FIELD_Y + GV.dn.y,
                            1.0, bullet.angle + Math.PI / 2.0,
                            imageManager.img_bullet[bullet.knd, bullet.col], 1);

                        if (bullet.eff == 1)
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                    }
                }
            }
            DX.SetDrawMode(DX.DX_DRAWMODE_NEAREST); // 描画形式を戻す
        }

        /// <summary>
        /// ボードを描画します。
        /// </summary>
        private void graph_board()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.DrawGraph(0, 0, imageManager.img_board[10], 0);
            DX.DrawGraph(0, 16, imageManager.img_board[11], 0);
            DX.DrawGraph(0, 464, imageManager.img_board[12], 0);
            DX.DrawGraph(416, 0, imageManager.img_board[20], 0);
        }

        /// <summary>
        /// 開発用の文字を描画します。
        /// </summary>
        private void graph_develop()
        {
            DX.DrawString(0, 0, "" + GV.stage_count, DX.GetColor(255, 255, 255), DX.GetColor(255, 255, 255));
        }

        /// <summary>
        /// 描画します。
        /// </summary>
        public void graph_main()
        {
            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            this.graph_back_main(); // 背景描画メイン
            this.graph_effect(0); // 敵が死ぬエフェクト

            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_effect(4); // 喰らいボムのエフェクト

            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            this.graph_boss(); // ボスの描画
            this.graph_enemy(); // 敵の描画
            this.graph_cshot(); // 自キャラクタのショットの描画

            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_ch(); // 自キャラクタの描画

            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            this.graph_bullet(); // 弾の描画
            
            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_effect(1); // ボムのエフェクト
            this.graph_effect(2); // ボム線のエフェクト
            this.graph_effect(3); // ボムキャラのエフェクト
            this.graph_board(); // ボードの描画

            this.graph_develop(); // 開発用の描画
        }
    }
}
