﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 自キャラクタのショットを表すクラスです。
    /// </summary>
    class CShot
    {
        /// <summary>
        /// 敵の当たり判定範囲。
        /// TODO 暫定でここに置きます。
        /// </summary>
        private int[] enemy_range = new int[] { 16, 30, 16, 50 };

        /// <summary>
        /// 自キャラクタのショットの当たり判定範囲。
        /// </summary>
        private int[] cshot_range = new int[] { 6, 0 };

        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// パワー。
        /// </summary>
        public int power;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;
        
        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// 角度。
        /// </summary>
        public double angle;

        /// <summary>
        /// スピード。
        /// </summary>
        public double spd;
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.power = 0;
            this.cnt = 0;
            this.knd = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.angle = 0.0;
            this.spd = 0.0;
        }
        
        /// <summary>
        /// 敵に当たったかどうか判定する。
        /// </summary>
        /// <param name="enemy">敵</param>
        /// <returns>当たった場合は true</returns>
        private bool out_judge_cshot(Enemy enemy)
        {
            if (this.cnt > 0) // ショットの軌道が一度でも計算されていたら
            {
                return MyMath.out_judge(this.x, this.y, enemy.x, enemy.y, cshot_range[this.knd], enemy_range[enemy.knd], this.spd, this.angle);
            }
            return false;
        }

        /// <summary>
        /// ボスに当たったかどうか判定する。
        /// </summary>
        /// <param name="enemy">ボス</param>
        /// <returns>当たった場合は true</returns>
        private bool out_judge_cshot_boss(Boss boss)
        {
            if (this.cnt > 0) // ショットの軌道が一度でも計算されていたら
            {
                return MyMath.out_judge(this.x, this.y, boss.x, boss.y, cshot_range[this.knd], Setting.BRANGE, this.spd, this.angle);
            }
            return false;
        }

        /// <summary>
        /// 自キャラクタのショットが敵に当たる処理をします。
        /// </summary>
        public void cshot_and_enemy()
        {
            for (int s = 0; s < GV.enemys.length(); s++) // 敵の総数
            {
                Enemy enemy = GV.enemys[s];
                if (enemy.flag > 0)
                {
                    if (this.out_judge_cshot(enemy)) // 自キャラクタのショットと敵が当たっていたら
                    {
                        this.flag = 0; // 自キャラクタのショットを消す
                        enemy.hit_enemy(this.power); // 敵に対して処理する
                        return; // 消したショットでボスを判定しないために原作では break; のところを return; にする
                    }
                }
            }

            // ボスが出現していて、描画しないフラグがオフで、ショット中なら
            if (GV.boss.flag == 1 && GV.boss.graph_flag == 0 && GV.boss.state == 2)
            {
                if(this.out_judge_cshot_boss(GV.boss))
                {
                    this.flag = 0; // 自キャラクタのショットを消す
                    GV.boss.hit_boss(this.power); // ボスに対して処理する
                }
            }
        }
    }
}
