﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// グラフィックスを管理するクラスです。
    /// </summary>
    class GraphManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static GraphManager manager = new GraphManager();

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private GraphManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static GraphManager GetInstance()
        {
            return manager;
        }
        
        /// <summary>
        /// 背景番号 0。通常の背景を描画します。
        /// </summary>
        private void graph_back_00()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawArea(Setting.FX, Setting.FY, Setting.FX + Setting.FMX, Setting.FY + Setting.FMY); // 描画可能エリアを設定
            DX.DrawGraph(Setting.FX, GV.count % 700 + Setting.FY - 700, imageManager.img_back[0], 0);
            DX.DrawGraph(Setting.FX, GV.count % 700 + Setting.FY, imageManager.img_back[0], 0);
            DX.SetDrawArea(0, 0, 640, 480); // エリアを戻す
        }

        /// <summary>
        /// 背景番号 1。スペルカード用の背景を描画します。
        /// </summary>
        private void graph_back_01()
        {
            DX.SetDrawArea(Setting.FX, Setting.FY, Setting.FX + Setting.FMX, Setting.FY + Setting.FMY); // 描画可能エリアを設定

            // ここにスペルカード用の背景を描く

            DX.SetDrawArea(0, 0, 640, 480); // エリアを戻す
        }

        /// <summary>
        /// 背景を描画するメイン処理です。
        /// </summary>
        private void graph_back_main()
        {
            // ボスがオンでスペルカードなら
            if(GV.boss.flag == 1 && GV.boss.back_knd[GV.boss.knd] == 1)
                this.graph_back_01();
            else
                this.graph_back_00();
        }

        /// <summary>
        /// エフェクト群を描画します。
        /// </summary>
        /// <param name="knd">エフェクトの種類</param>
        private void graph_effect(int knd)
        {
            for (int i = 0; i < GV.effects.length(); i++)
            {
                Effect effect = GV.effects[i];
                if (effect.flag > 0 && effect.knd == knd)
                {
                    if (effect.eff == 1) // エフェクトが光エフェクトなら
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, (int)effect.brt);
                    if (effect.eff == 2) // エフェクトがαエフェクトなら
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_ALPHA, (int)effect.brt);

                    DX.DrawRotaGraphF((float)effect.x + Setting.FX + GV.dn.x, (float)effect.y + Setting.FY + GV.dn.y,
                        effect.r, effect.ang, effect.img, 1);

                    if (effect.eff == 1 || effect.eff == 2)
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                }
            }
        }

        /// <summary>
        /// ボスのエフェクトを描画します。
        /// </summary>
        private void graph_boss_effect()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ALPHA, 150);

            // ボス背景の円形の内側
            DX.DrawRotaGraphF(
                (float)GV.boss.dx + Setting.FX,
                (float)GV.boss.dy + Setting.FY,
                (0.4 + 0.05 * Math.Sin(2 * Math.PI / 360 * (GV.count % 360))) * 3,
                MyMath.PI2 * (GV.count % 580) / 580,
                imageManager.img_etc[5], 1);
            // ボス背景の円形の外側
            DX.DrawRotaGraphF(
                (float)GV.boss.dx + Setting.FX,
                (float)GV.boss.dy + Setting.FY,
                (0.5 + 0.1 * Math.Sin(MyMath.PI2 / 360 * (GV.count % 360))) * 2,
                2 * Math.PI * (GV.count % 340) / 340,
                imageManager.img_etc[2], 1);

            // 奥にうっすら表示される回転体
            DX.DrawRotaGraphF(
                (float)(GV.boss.dx + 60 * Math.Sin(MyMath.PI2 / 153 * (GV.count % 153))) + Setting.FX,
                (float)(GV.boss.dy + 80 * Math.Sin(MyMath.PI2 / 120 * (GV.count % 120))) + Setting.FY,
                0.4 + 0.05 * Math.Sin(MyMath.PI2 / 120 * (GV.count % 120)),
                2 * Math.PI * (GV.count % 30) / 30,
                imageManager.img_etc[6], 1);
            DX.DrawRotaGraphF(
                (float)(GV.boss.dx + 60 * Math.Sin(MyMath.PI2 / 200 * ((GV.count + 20) % 200))) + Setting.FX,
                (float)(GV.boss.dy + 80 * Math.Sin(MyMath.PI2 / 177 * ((GV.count + 20) % 177))) + Setting.FY,
                0.3 + 0.05 * Math.Sin(MyMath.PI2 / 120 * (GV.count % 120)),
                2 * Math.PI * (GV.count % 35) / 35,
                imageManager.img_etc[6], 1);

            DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);

            // 手前に表示される回転体
            DX.DrawRotaGraphF(
                (float)(GV.boss.dx + 60 * Math.Sin(MyMath.PI2 / 230 * ((GV.count + 40) % 230))) + Setting.FX,
                (float)(GV.boss.dy + 80 * Math.Sin(MyMath.PI2 / 189 * ((GV.count + 40) % 189))) + Setting.FY,
                0.6 + 0.05 * Math.Sin(MyMath.PI2 / 120 * (GV.count % 120)),
                2 * Math.PI * (GV.count % 40) / 40,
                imageManager.img_etc[6], 1);
        }

        /// <summary>
        /// ボスを描画します。
        /// </summary>
        private void graph_boss()
        {
            if (GV.boss.flag == 0)
                return;

            this.graph_boss_effect(); // ボスのエフェクト描画する

            ImageManager imageManager = ImageManager.GetInstance();
            DX.DrawRotaGraphF((float)GV.boss.dx + Setting.FX + GV.dn.x, (float)GV.boss.dy + Setting.FY + GV.dn.y, 1.0, 0.0, imageManager.img_dot_riria[0], 1);
            if (GV.boss.hp_max == 0)
            {
                Debug.WriteLine("graph_boss内 0割り");
                return;
            }
            for (int i = 0; i < Setting.FMX * 0.98 * GV.boss.hp / GV.boss.hp_max; i++) // HPを表示する
            {
                if (GV.boss.back_knd[GV.boss.knd] == 1)
                    DX.DrawGraph(3 + Setting.FX + i + GV.dn.x, 2 + Setting.FY + GV.dn.y, imageManager.img_etc[7], 0);
                else
                    DX.DrawGraph(3 + Setting.FX + i + GV.dn.x, 2 + Setting.FY + GV.dn.y, imageManager.img_etc[1], 0);
            }
        }

        /// <summary>
        /// 敵キャラクタ群を描画します。
        /// </summary>
        private void graph_enemy()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.enemys.length(); i++)
            {
                Enemy enemy = GV.enemys[i];
                if (enemy.flag == 1)
                    DX.DrawRotaGraphF((float)enemy.x + Setting.FX + GV.dn.x, (float)enemy.y + Setting.FY + GV.dn.y, 1.0, 0.0, imageManager.img_enemy[0, enemy.img], 1);
            }
        }

        /// <summary>
        /// ショットを描画します。
        /// </summary>
        private void graph_cshot()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.cshots.length(); i++)
            {
                CShot cshot = GV.cshots[i];
                if (cshot.flag > 0)
                    DX.DrawRotaGraphF((float)cshot.x + Setting.FX + GV.dn.x, (float)cshot.y + Setting.FY + GV.dn.y, 1.0, 0.0, imageManager.img_cshot[cshot.knd], 1);
            }
        }

        /// <summary>
        /// 自キャラクタを描画します。
        /// </summary>
        private void graph_ch()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            KeyManager keyManager = KeyManager.GetInstance();
            double sx;
            double sy;
            double ny = Math.Sin(2.0 * Math.PI * (GV.count % 50) / 50) * 3;
            double ang = 2.0 * Math.PI * (GV.count % 120) / 120;

            if (keyManager.CheckStatePad(GV.configpad.slow) > 0) // 低速移動中なら
            {
                sx = 15; // 引き寄せる
                sy = 15 + ny;
            }
            else
            {
                sx = 30; // 普通の位置に
                sy = 30 + ny;
            }

            // ボールの表示(dn.x, dn.yの加算はないけどOK?)
            DX.DrawRotaGraphF((float)(GV.ch.x - sx) + Setting.FX, (float)(GV.ch.y + sy) + Setting.FY + GV.dn.y, 1.0, ang, imageManager.img_chetc[2], 1);
            DX.DrawRotaGraphF((float)(GV.ch.x + sx) + Setting.FX, (float)(GV.ch.y + sy) + Setting.FY + GV.dn.y, 1.0, -ang, imageManager.img_chetc[2], 1);

            if (GV.ch.mutekicnt % 2 == 0) // 無敵中なら点滅
            {
                // 自キャラクタを表示
                DX.DrawRotaGraphF((float)GV.ch.x + Setting.FX + GV.dn.x, (float)GV.ch.y + Setting.FY + GV.dn.y, 1.0, 0.0, imageManager.img_ch[0, GV.ch.img], 1);
                if(keyManager.CheckStatePad(GV.configpad.slow) > 0) // 移動低速中なら当たり判定表示(dn.x, dn.yの加算はないけどOK?)
                    DX.DrawRotaGraphF((float)GV.ch.x + Setting.FX, (float)GV.ch.y + Setting.FY, 1.0, 2.0 * Math.PI * (GV.count % 120) / 120, imageManager.img_chetc[0], 1);
            }
        }

        /// <summary>
        /// 弾幕を描画します。
        /// </summary>
        private void graph_bullet()
        {
            double disp_angle; // 表示角度

            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawMode(DX.DX_DRAWMODE_BILINEAR); // 線形補間描画

            // 敵弾
            for (int i = 0; i < GV.shots.length(); i++) // 弾幕数分ループ
            {
                Shot shot = GV.shots[i];
                if (shot.flag > 0) // 弾幕データがオンなら
                {
                    BulletGroup bullets = shot.bullets;
                    for (int j = 0; j < bullets.length(); j++) // 弾幕が持つの弾の数分ループ
                    {
                        Bullet bullet = bullets[j];
                        if (bullet.flag != 0) // 弾データがオンなら
                        {
                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);

                            if (GV.bullet_infos[bullet.knd].kaiten == 1) // 弾の種類が回転する弾の場合
                                disp_angle = MyMath.PI2 * (bullet.cnt % 120) / 120;
                            else
                                disp_angle = bullet.angle + Math.PI / 2;

                            DX.DrawRotaGraphF((float)bullet.x + Setting.FX + GV.dn.x, (float)bullet.y + Setting.FY + GV.dn.y,
                                1.0, disp_angle,
                                imageManager.img_bullet[bullet.knd, bullet.col], 1);

                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                        }
                    }
                }
            }

            // ボス弾
            if(GV.boss_shot.flag > 0) // 弾幕データがオンなら
            {
                BulletGroup bullets = GV.boss_shot.bullets;
                for (int j = 0; j < bullets.length(); j++) // 弾幕が持つの弾の数分ループ
                {
                    Bullet bullet = bullets[j];
                    if (bullet.flag != 0) // 弾データがオンなら
                    {
                        if (bullet.eff == 1)
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);

                        if (bullet.kaiten == 1) // 弾が回転状態なら
                            disp_angle = MyMath.PI2 * (bullet.cnt % 120) / 120;
                        else
                            disp_angle = bullet.angle + Math.PI / 2;

                        int x = DX.DrawRotaGraphF((float)bullet.x + Setting.FX + GV.dn.x, (float)bullet.y + Setting.FY + GV.dn.y,
                            1.0, disp_angle,
                            imageManager.img_bullet[bullet.knd, bullet.col], 1);

                        if (x != 0)
                            Debug.WriteLine("a");

                        if (bullet.eff == 1)
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                    }
                }
            }
            DX.SetDrawMode(DX.DX_DRAWMODE_NEAREST); // 描画形式を戻す
        }

        /// <summary>
        /// ボードを描画します。
        /// </summary>
        private void graph_board()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.DrawGraph(0, 0, imageManager.img_board[10], 0);
            DX.DrawGraph(0, 16, imageManager.img_board[11], 0);
            DX.DrawGraph(0, 464, imageManager.img_board[12], 0);
            DX.DrawGraph(416, 0, imageManager.img_board[20], 0);
        }

        /// <summary>
        /// 開発用の文字を描画します。
        /// </summary>
        private void graph_develop()
        {
            DX.DrawString(0, 0, "" + GV.stage_count, DX.GetColor(255, 255, 255), DX.GetColor(255, 255, 255));
        }

        /// <summary>
        /// 描画します。
        /// </summary>
        public void graph_main()
        {
            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            this.graph_back_main(); // 背景描画メイン
            this.graph_effect(0); // 敵が死ぬエフェクト

            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_effect(4); // 喰らいボムのエフェクト

            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            this.graph_boss(); // ボスの描画
            this.graph_enemy(); // 敵の描画
            this.graph_cshot(); // 自キャラクタのショットの描画

            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_ch(); // 自キャラクタの描画

            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            this.graph_bullet(); // 弾の描画
            
            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_effect(1); // ボムのエフェクト
            this.graph_effect(2); // ボム線のエフェクト
            this.graph_effect(3); // ボムキャラのエフェクト
            this.graph_board(); // ボードの描画

            this.graph_develop(); // 開発用の描画
        }
    }
}
