﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボスの弾幕を表すクラスです。
    /// </summary>
    class BossShot
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;
        
        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// どの敵から発射されたのか。
        /// </summary>
        public Enemy parent_enemy;

        /// <summary>
        /// ベース角度。
        /// </summary>
        public double[] base_angle = new double[10];

        /// <summary>
        /// ベーススピード。
        /// </summary>
        public double[] base_spd = new double[1];

        /// <summary>
        /// 弾群。
        /// </summary>
        public BulletGroup bullets = new BulletGroup(Setting.BOSS_BULLET_MAX);
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.knd = 0;
            this.cnt = 0;
            this.parent_enemy = null;
            for (int i = 0; i < this.base_angle.Length; i++)
                this.base_angle[i] = 0;
            for (int i = 0; i < this.base_spd.Length; i++)
                this.base_spd[i] = 0;
            this.bullets.initialize();
        }

        /// <summary>
        /// 空いている弾を返します。
        /// </summary>
        /// <returns>弾。無い場合には null</returns>
        public Bullet search_boss_shot()
        {
            return this.bullets.search_unused_bullet();
        }

        /// <summary>
        /// ボスの弾幕を計算します。
        /// </summary>
        public void boss_shot_calc()
        {
            GV.boss.decrease_endtime();
            this.bullets.calc(); // ボスの弾幕の計算は敵の弾幕の計算そのまま

            this.cnt++;
        }

        /// <summary>
        /// 弾幕をセットします。
        /// </summary>
        public void enter_boss_shot()
        {
            this.initialize(); // ショット・弾情報を初期化
            this.flag = 1;
        }

        /// <summary>
        /// ボスショットと自キャラクタとの処理をします。
        /// </summary>
        public void bossshot_and_ch()
        {
            if (this.flag > 0) // ショットが登録されていれば
                this.bullets.enemyshot_and_ch();
        }
        
        /// <summary>
        /// 弾幕を処理します。必要に応じて弾を追加します。
        /// </summary>
        public void do_boss_shot_bullet()
        {
            BossShotPattern.do_boss_shot_bullet(this);
        }
    }
}
