﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// 音楽を管理するためのクラスです。
    /// </summary>
    class MusicManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static MusicManager manager = new MusicManager();
        
        /// <summary>
        /// サウンドSE。
        /// </summary>
        private int[] sound_se = new int[Setting.SE_MAX];

        /// <summary>
        /// サウンドSEフラグ。
        /// </summary>
        public int[] se_flag = new int[Setting.SE_MAX];

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private MusicManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static MusicManager GetInstance()
        {
            return manager;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void ini()
        {
            for (int i = 0; i < this.se_flag.Length; i++)
                this.se_flag[i] = 0;
        }

        /// <summary>
        /// 音楽を鳴らします。
        /// </summary>
        public void play()
        {
            for (int i = 0; i < this.sound_se.Length; i++)
                if (this.se_flag[i] == 1)
                {
                    if(DX.CheckSoundMem(this.sound_se[i]) != 0)
                    {
                        /* ショット音の関係で暫定的にコメントアウト
                        if(i == 8) // 8番はショットが敵に当たった音の場合には
                            continue; // 音楽を止めない
                         */
                        DX.StopSoundMem(this.sound_se[i]);
                    }
                    DX.PlaySoundMem(this.sound_se[i], DX.DX_PLAYTYPE_BACK);
                }
        }
        
        /// <summary>
        /// 音楽データを読み込みます。
        /// </summary>
        public void load()
        {
            string baseDir = Setting.BASE_DIR;
            this.sound_se[0] = DX.LoadSoundMem(baseDir + "se/enemy_shot.wav");
            this.sound_se[1] = DX.LoadSoundMem(baseDir + "se/enemy_death.wav");
            this.sound_se[2] = DX.LoadSoundMem(baseDir + "se/cshot.wav");
            this.sound_se[3] = DX.LoadSoundMem(baseDir + "se/char_death.wav"); // 自キャラクタの死に音
            this.sound_se[8] = DX.LoadSoundMem(baseDir + "se/hit.wav");
            this.sound_se[14] = DX.LoadSoundMem(baseDir + "se/bom1.wav");
            this.sound_se[15] = DX.LoadSoundMem(baseDir + "se/bom2.wav");

            // 各素材の再生ボリュームを設定
            DX.ChangeVolumeSoundMem(50, this.sound_se[0]);
            DX.ChangeVolumeSoundMem(128, this.sound_se[1]);
            DX.ChangeVolumeSoundMem(128, this.sound_se[2]);
            DX.ChangeVolumeSoundMem(50, this.sound_se[8]);
        }
    }
}
