﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// グローバル変数や関数を管理するためのクラスです。
    /// </summary>
    class GV
    {
        /// <summary>
        /// 自キャラクタ。
        /// </summary>
        public static Ch ch = new Ch();
        
        /// <summary>
        /// 自キャラクタのショット群。
        /// </summary>
        public static CShotGroup cshots = new CShotGroup();

        /// <summary>
        /// 敵キャラクタ群。
        /// </summary>
        public static EnemyGroup enemys = new EnemyGroup();

        /// <summary>
        /// 敵の出現情報群。
        /// </summary>
        public static EnemyOrderGroup enemy_orders = new EnemyOrderGroup();
        
        /// <summary>
        /// 敵の弾幕群。
        /// </summary>
        public static ShotGroup shots = new ShotGroup();
        
        /// <summary>
        /// エフェクト群。
        /// </summary>
        public static EffectGroup effects = new EffectGroup();

        /// <summary>
        /// 消滅エフェクト群。
        /// </summary>
        public static DelEffectGroup del_effects = new DelEffectGroup();

        /// <summary>
        /// 弾の当たり判定情報群。
        /// </summary>
        public static BulletInfoGroup bullet_infos = new BulletInfoGroup();
        
        /// <summary>
        /// ボム。
        /// </summary>
        public static Bom bom = new Bom();

        /// <summary>
        /// 画面の輝度。
        /// </summary>
        public static BrightSet bright_set = new BrightSet();

        /// <summary>
        /// ドン揺れ。
        /// </summary>
        public static Dn dn = new Dn();

        /// <summary>
        /// ボスショット。
        /// </summary>
        public static BossShot boss_shot = new BossShot();

        /// <summary>
        /// ボス。
        /// </summary>
        public static Boss boss = new Boss();

        /// <summary>
        /// レーザー群。
        /// </summary>
        public static LazerGroup lazers = new LazerGroup();

        /// <summary>
        /// 子ども群。
        /// </summary>
        public static ChildGroup children = new ChildGroup();

        /// <summary>
        /// メインメソッド状態。
        /// </summary>
        public static int func_state;
        
        /// <summary>
        /// コンフィグ。
        /// </summary>
        public static ConfigPad configpad = new ConfigPad();

        /// <summary>
        /// ゲーム中のカウンタ。
        /// </summary>
        public static int stage_count;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public static int count;

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private GV()
        {
            ;  //何もしない
        }

        /// <summary>
        /// 最初の初期化処理をします。
        /// </summary>
        public static void first_ini()
        {
            configpad.first_ini(); // コンフィグの初期化
        }

        /// <summary>
        /// ゲームの初期化をします。
        /// </summary>
        public static void ini()
        {
            stage_count = 1;

            ch.initialize();
            enemys.initialize();
            lazers.initialize();
            enemy_orders.initialize();
            shots.initialize();
            cshots.initialize();
            effects.initialize();
            del_effects.initialize();
            bom.initialize();
            bright_set.initialize();
            dn.initialize();
            boss.initialize();
            children.initialize();

            ch.ini(); // 自キャラクタの初期化
            boss.ini();
            bright_set.ini();
        }
        
        /// <summary>
        /// カウンタを増やします。
        /// </summary>
        public static void incrementCount()
        {
            count++;
        }

        /// <summary>
        /// ステージカウンタを増やします。
        /// </summary>
        public static void increment_stage_count()
        {
            stage_count++;
        }
    }
}
