﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// アイテム群を管理するクラスです。
    /// </summary>
    class ItemGroup
    {
        /// <summary>
        /// アイテム群。
        /// </summary>
        private Item[] items = new Item[Setting.ITEM_MAX];

        /// <summary>
        /// アイテム画像の拡大率。
        /// </summary>
        private static double[] r = new double[] { 0.6, 0.6, 1.0, 0.6, 1.0, 1.0};

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public ItemGroup()
        {
            for (int i = 0; i < this.items.Length; i++)
            {
                this.items[i] = new Item();
            }
        }

        /// <summary>
        /// アイテムにアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>アイテム</returns>
        public Item this[int index]
        {
            get
            {
                return this.items[index];
            }
        }

        /// <summary>
        /// アイテムの数を返します。
        /// </summary>
        /// <returns>アイテムの数</returns>
        public int length()
        {
            return this.items.Length;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.items.Length; i++)
                this.items[i].initialize();
        }

        /// <summary>
        /// 空いているアイテムを返します。
        /// </summary>
        /// <returns>アイテム。空きが無い場合は null</returns>
        private Item search_item()
        {
            for (int i = 0; i < this.items.Length; i++)
            {
                Item item = this.items[i];
                if (item.flag == 0)
                    return item;
            }

            return null;
        }

        /// <summary>
        /// アイテムを登録します。
        /// </summary>
        /// <param name="x">X座標</param>
        /// <param name="y">Y座標</param>
        /// <param name="item_n">アイテム群</param>
        public void enter_item(double x, double y, int[] item_n)
        {
            for (int i = 0; i < item_n.Length; i++)
            {
                if (item_n[i] != -1) // アイテムを登録する必要があるなら(CSVファイルで -1以外が指定されていたなら)
                {
                    Item item = this.search_item(); // 空いているアイテムを探す
                    if (item != null)
                    {
                        item.flag = 1;
                        item.v = -3.5; // スピード
                        item.cnt = 0;
                        item.state = 0;
                        item.x = x;
                        item.y = y;

                        if (i > 0) // 複数なら 2個目以降を適当にちらばせる
                        {
                            item.x += MyMath.rang(40);
                            item.y += MyMath.rang(40);
                        }

                        item.knd = item_n[i]; // 0:小パワー, 1:小点, 2:弾点, 3:小金, 4:大パワー, 5:大金
                        item.r = r[item.knd];
                    }
                }
            }
        }

        /// <summary>
        /// アイテム群を計算します。
        /// </summary>
        public void calc_item()
        {
            for (int i = 0; i < this.items.Length; i++)
            {
                Item item = this.items[i];
                if (item.flag > 0)
                    item.calc_item();
            }
        }
    }
}
