﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// レーザーを表すクラスです。
    /// </summary>
    class Lazer
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;
        
        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// 色。
        /// </summary>
        public int col;

        /// <summary>
        /// 状態。
        /// </summary>
        public int state;

        /// <summary>
        /// 幅。
        /// </summary>
        public double haba;

        /// <summary>
        /// 角度。
        /// </summary>
        public double angle;

        /// <summary>
        /// 長さ。
        /// </summary>
        public double length;

        /// <summary>
        /// 判定範囲(表示幅に対して 0～1で指定)
        /// </summary>
        public double hantei;

        /// <summary>
        /// レーザーを発射する始点。
        /// </summary>
        public Pt startpt = new Pt();

        /// <summary>
        /// レーザーの表示座標。
        /// </summary>
        public Pt[] disppts = new Pt[4];

        /// <summary>
        /// レーザーの当たり判定範囲。
        /// </summary>
        public Pt[] outpts = new Pt[4];

        /// <summary>
        /// レーザーの物理演算。
        /// </summary>
        public LPhy lphy = new LPhy();

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public Lazer()
        {
            for (int i = 0; i < disppts.Length; i++)
                this.disppts[i] = new Pt();
            for (int i = 0; i < outpts.Length; i++)
                this.outpts[i] = new Pt();
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.knd = 0;
            this.col = 0;
            this.state = 0;
            this.haba = 0.0;
            this.angle = 0.0;
            this.length = 0.0;
            this.hantei = 0.0;
            this.startpt.initialize();
            for (int i = 0; i < this.disppts.Length; i++)
                this.disppts[i].initialize();
            for (int i = 0; i < this.outpts.Length; i++)
                this.outpts[i].initialize();
            this.lphy.initialize();
        }

        /// <summary>
        /// レーザーの物理演算を登録します。
        /// </summary>
        /// <param name="time">時間</param>
        /// <param name="angle">角度</param>
        public void input_lphy(int time, double angle)
        {
            this.lphy.input_lphy(this, time, angle);
        }

        /// <summary>
        /// レーザーを計算します。
        /// </summary>
        public void lazer_calc()
        {
            // 表示位置を設定
            this.disppts[0].x = this.startpt.x + Math.Cos(this.angle + Math.PI / 2) * this.haba;
            this.disppts[0].y = this.startpt.y + Math.Sin(this.angle + Math.PI / 2) * this.haba;
            this.disppts[1].x = this.startpt.x + Math.Cos(this.angle - Math.PI / 2) * this.haba;
            this.disppts[1].y = this.startpt.y + Math.Sin(this.angle - Math.PI / 2) * this.haba;
            this.disppts[2].x = this.startpt.x + Math.Cos(this.angle - Math.PI / 2) * this.haba + Math.Cos(this.angle) * this.length;
            this.disppts[2].y = this.startpt.y + Math.Sin(this.angle - Math.PI / 2) * this.haba + Math.Sin(this.angle) * this.length;
            this.disppts[3].x = this.startpt.x + Math.Cos(this.angle + Math.PI / 2) * this.haba + Math.Cos(this.angle) * this.length;
            this.disppts[3].y = this.startpt.y + Math.Sin(this.angle + Math.PI / 2) * this.haba + Math.Sin(this.angle) * this.length;

            // あたり範囲を設定
            this.outpts[0].x = this.startpt.x + Math.Cos(this.angle + Math.PI / 2) * (this.haba * this.hantei) + Math.Cos(this.angle) * this.length * ((1 - this.hantei) / 2);
            this.outpts[0].y = this.startpt.y + Math.Sin(this.angle + Math.PI / 2) * (this.haba * this.hantei) + Math.Sin(this.angle) * this.length * ((1 - this.hantei) / 2);
            this.outpts[1].x = this.startpt.x + Math.Cos(this.angle - Math.PI / 2) * (this.haba * this.hantei) + Math.Cos(this.angle) * this.length * ((1 - this.hantei) / 2);
            this.outpts[1].y = this.startpt.y + Math.Sin(this.angle - Math.PI / 2) * (this.haba * this.hantei) + Math.Sin(this.angle) * this.length * ((1 - this.hantei) / 2);
            this.outpts[2].x = this.startpt.x + Math.Cos(this.angle - Math.PI / 2) * (this.haba * this.hantei) + Math.Cos(this.angle) * this.length * this.hantei + Math.Cos(this.angle) * this.length * ((1 - this.hantei) / 2);
            this.outpts[2].y = this.startpt.y + Math.Sin(this.angle - Math.PI / 2) * (this.haba * this.hantei) + Math.Sin(this.angle) * this.length * this.hantei + Math.Sin(this.angle) * this.length * ((1 - this.hantei) / 2);
            this.outpts[3].x = this.startpt.x + Math.Cos(this.angle + Math.PI / 2) * (this.haba * this.hantei) + Math.Cos(this.angle) * this.length * this.hantei + Math.Cos(this.angle) * this.length * ((1 - this.hantei) / 2);
            this.outpts[3].y = this.startpt.y + Math.Sin(this.angle + Math.PI / 2) * (this.haba * this.hantei) + Math.Sin(this.angle) * this.length * this.hantei + Math.Sin(this.angle) * this.length * ((1 - this.hantei) / 2);

            double ymax = this.lphy.angle;
            double ty = this.lphy.time;
            int t = this.cnt;
            double delt = (2 * ymax * t / ty - ymax * t * t / (ty * ty));

            if (this.lphy.time != 0)// 回転移動時間内なら
                this.angle = this.lphy.base_ang + delt; // 回転する

            if (this.lphy.conv_flag == 1) //座標変換をするか
            {
                this.startpt.conv_pos0(this.lphy.conv_x, this.lphy.conv_y,
                    this.lphy.conv_base_x, this.lphy.conv_base_y, -delt);
            }
            if (this.cnt > this.lphy.time) // 回転時間を過ぎると
            {
                this.lphy.time = 0;
                this.lphy.conv_flag = 0; // 回転をやめる
            }
            this.cnt++;
        }

        /// <summary>
        /// レーザーの処理をします。
        /// </summary>
        /// <param name="sqrpt">4点の座標</param>
        /// <param name="rpt">自キャラクタの座標</param>
        /// <returns>レーザーに接触した場合は true</returns>
        public bool out_lazer(Pt[] sqrpt, Pt rpt)
        {
            if (this.hantei != 0)
            {
                for (int j = 0; j < 4; j++) // レーザーの 4点を設定
                {
                    sqrpt[j].x = this.outpts[j].x;
                    sqrpt[j].y = this.outpts[j].y;
                }

                // 長方形と円との接触判定
                if (rpt.hitjudge_square_and_circle(sqrpt, Setting.CRANGE))
                    return true;
            }

            return false;
        }
    }
}
