﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// アイテムを表します。
    /// </summary>
    class Item
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// 状態。
        /// </summary>
        public int state;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;
        
        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;
        
        /// <summary>
        /// 速度。
        /// </summary>
        public double v;

        /// <summary>
        /// 大きさ。
        /// </summary>
        public double r;

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.knd = 0;
            this.state = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.v = 0.0;
            this.r = 0.0;
        }

        /// <summary>
        /// アイテムの吸収処理をします。
        /// </summary>
        private void calc_item_indraw()
        {
            double v = (this.state == 1) ? 8 : 3; // stateが 1ならスピード 8、違うなら 2
            double angle = Math.Atan2(GV.ch.y - this.y, GV.ch.x - this.x); // 自機への角度を計算
            this.x += Math.Cos(angle) * v;
            this.y += Math.Sin(angle) * v;
        }

        /// <summary>
        /// アイテムを計算します。
        /// </summary>
        public void calc_item()
        {
            if (this.state == 0) // 普通の状態なら
                if (GV.ch.y < Setting.ITEM_GET_BORDER_LINE) // 自キャラクタが上方にいたら
                    this.state = 1; // 吸収状態にする

            if (this.state == 0) // 普通の状態なら
            {
                double x = GV.ch.x - this.x;
                double y = GV.ch.y - this.y;
                KeyManager keyManager = KeyManager.GetInstance();

                //低速状態で自機付近なら
                if (keyManager.CheckStatePad(GV.configpad.slow) > 0
                    && x * x + y * y < Setting.ITEM_INDRAW_RANGE * Setting.ITEM_INDRAW_RANGE)
                {
                    this.calc_item_indraw();
                }
                else
                {
                    if (this.v < 2.5)
                        this.v += 0.06; // 速度アップ
                    this.y += this.v; // 移動する
                }
            }
            else // 吸収状態なら
                this.calc_item_indraw();

            this.cnt++;

            if (this.y > Setting.FMY + 50) // 画面外に出たら
                this.flag = 0; // 消滅する
        }

        /// <summary>
        /// アイテム接触処理をします。
        /// </summary>
        public void ch_and_item()
        {
            double x = this.x - GV.ch.x;
            double y = this.y - GV.ch.y;

            if (x * x + y * y < Setting.ITEM_HIT_BOX * Setting.ITEM_HIT_BOX)
            {
                switch (this.knd)
                {
                    case 0: // 小パワー
                        GV.ch.addPower(3);
                        break;
                    case 1: // 小点
                        GV.ch.addPoint(1);
                        break;
                    case 2: // 弾点
                        GV.ch.addScore(1);
                        break;
                    case 3: // 小金
                        GV.ch.addMoney(1);
                        break;
                    case 4: // 大パワー
                        GV.ch.addPower(50);
                        break;
                    case 5: // 大金
                        GV.ch.addMoney(10);
                        break;
                    default:
                        Debug.WriteLine("ch_and_itemメソッドで範囲外の knd(" + this.knd + ")");
                        break;
                }
                this.flag = 0;

                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[34] = 1;
            }
        }
    }
}
