﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// レーザー群を管理するクラスです。
    /// </summary>
    class LazerGroup
    {
        /// <summary>
        /// レーザー群。
        /// </summary>
        private Lazer[] lazers = new Lazer[Setting.LAZER_MAX];

        /// <summary>
        /// 長方形の 4点。
        /// </summary>
        private Pt[] sqrp = new Pt[4];

        /// <summary>
        /// 自キャラクタの位置。
        /// </summary>
        private Pt rpt = new Pt();

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public LazerGroup()
        {
            for (int i = 0; i < this.lazers.Length; i++)
                this.lazers[i] = new Lazer();
            for (int i = 0; i < this.sqrp.Length; i++)
                this.sqrp[i] = new Pt();
        }

        /// <summary>
        /// レーザーにアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>レーザー</returns>
        public Lazer this[int index]
        {
            get
            {
                return this.lazers[index];
            }
        }

        /// <summary>
        /// レーザーの数を返します。
        /// </summary>
        /// <returns>レーザーの数</returns>
        public int length()
        {
            return this.lazers.Length;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.lazers.Length; i++)
                this.lazers[i].initialize();
        }

        /// <summary>
        /// 空いているレーザーを返します。
        /// </summary>
        /// <returns>レーザー。空きが無い場合は null</returns>
        public Lazer search_lazer()
        {
            for (int i = 0; i < this.lazers.Length; i++)
            {
                Lazer lazer = this.lazers[i];
                if (lazer.flag == 0)
                    return lazer;
            }

            return null;
        }

        /// <summary>
        /// レーザー群を処理します。
        /// </summary>
        private void lazer_calc()
        {
            for (int i = 0; i < this.lazers.Length; i++)
            {
                Lazer lazer = this.lazers[i];
                if (lazer.flag > 0)
                    lazer.lazer_calc();
            }
        }

        /// <summary>
        /// レーザーのメイン処理です。
        /// </summary>
        public void lazer_main()
        {
            this.lazer_calc();
        }

        /// <summary>
        /// レーザーに接触したかどうかを判定します。
        /// </summary>
        /// <returns>レーザに接触した場合には true</returns>
        private bool out_lazer()
        {
            this.rpt.x = GV.ch.x;
            this.rpt.y = GV.ch.y;

            for (int i = 0; i < sqrp.Length; i++)
                this.sqrp[i].initialize();

            for (int i = 0; i < this.lazers.Length; i++)
            {
                Lazer lazer = this.lazers[i];
                if (lazer.flag > 0)
                    if (lazer.out_lazer(this.sqrp, this.rpt)) // 接触したら
                        return true;
            }

            return false;
        }

        /// <summary>
        /// レーザーと自キャラクタの接触処理を行います。
        /// </summary>
        public void lazer_and_ch()
        {
            if (GV.ch.flag == 0 && GV.ch.mutekicnt == 0) // 生存中で無敵状態でもない場合に
            {
                if (this.out_lazer()) // レーザーに接触したら
                    GV.ch.death_by_lazer(); // 自キャラクタの死亡処理をする
            }
        }
    }
}
