﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 弾群を管理するクラスです。
    /// </summary>
    class BulletGroup
    {
        /// <summary>
        /// 弾群。
        /// </summary>
        private Bullet[] bullets;

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        /// <param name="num">弾数</param>
        public BulletGroup(int num)
        {
            this.bullets = new Bullet[num];
            for (int i = 0; i < this.bullets.Length; i++)
            {
                this.bullets[i] = new Bullet();
            }
        }

        /// <summary>
        /// 弾にアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>弾</returns>
        public Bullet this[int index]
        {
            get
            {
                return this.bullets[index];
            }
        }

        /// <summary>
        /// 弾の数を返します。
        /// </summary>
        /// <returns>弾の数</returns>
        public int length()
        {
            return this.bullets.Length;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.bullets.Length; i++)
                this.bullets[i].initialize();
        }

        /// <summary>
        /// 空いている弾を探します。
        /// </summary>
        /// <returns>弾。空いている弾が無い場合は null</returns>
        public Bullet search_unused_bullet()
        {
            for (int i = 0; i < this.bullets.Length; i++)
            {
                Bullet bullet = this.bullets[i];
                if (bullet.flag == 0)
                    return bullet; // 空いている弾を返す
            }

            return null; // 空きが無い場合は nullを返す
        }
        
        /// <summary>
        /// 弾群を計算します。
        /// </summary>
        public void calc()
        {
            for (int i = 0; i < this.bullets.Length; i++)
            {
                Bullet bullet = this.bullets[i];
                if (bullet.flag > 0) // その弾が登録されていたなら
                    bullet.calc(); // 弾について計算する
            }
        }

        /// <summary>
        /// 表示されている弾が存在するかどうか返します。
        /// </summary>
        /// <returns>弾がひとつでも存在する場合 true</returns>
        public bool exist_display_bullet()
        {
            for (int i = 0; i < this.bullets.Length; i++)
            {
                Bullet bullet = this.bullets[i];
                if (bullet.flag > 0)
                    return true;
            }

            return false;
        }

        /// <summary>
        /// 敵弾群と自キャラクタとの処理をします。
        /// </summary>
        /// <returns>自キャラクタが死んだ場合は true</returns>
        public bool enemyshot_and_ch()
        {
            for (int n = 0; n < this.bullets.Length; n++) // 弾総数
            {
                Bullet bullet = this.bullets[n];
                if (bullet.flag == 1) // 弾が登録されていたら
                {
                    bool isDead = bullet.enemyshot_and_ch();
                    if (isDead)
                        return true;
                }
            }

            return false;
        }
    }
}
