﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 敵の弾幕を表すクラスです。
    /// </summary>
    class Shot
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// どの敵から発射されたか。
        /// </summary>
        public Enemy parent_enemy;

        /// <summary>
        /// ベース角度。
        /// </summary>
        public double[] base_angle = new double[1];

        /// <summary>
        /// ベーススピード。
        /// </summary>
        public double[] base_spd = new double[1];

        /// <summary>
        /// 弾群。
        /// </summary>
        public BulletGroup bullets = new BulletGroup(Setting.SHOT_BULLET_MAX);

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.knd = 0;
            this.cnt = 0;
            this.parent_enemy = null;
            for (int i = 0; i < this.base_angle.Length; i++)
                this.base_angle[i] = 0;
            for (int i = 0; i < this.base_spd.Length; i++)
                this.base_spd[i] = 0;
            this.bullets.initialize();
        }

        /// <summary>
        /// 弾幕を処理します。
        /// </summary>
        public void do_shot_bullet()
        {
            ShotPattern.do_shot_bullet(this);
        }
        
        /// <summary>
        /// 弾幕を計算します。
        /// </summary>
        public void calc()
        {
            if (this.parent_enemy.flag != 1) // 敵が倒されたら
                this.flag = 2; // それ以上弾を登録しないフラグに変える
            this.bullets.calc(); // 弾群について計算する

            if (this.bullets.exist_display_bullet()) // 表示中の弾がひとつでもあれば
                return; // 処理を終える

            // 表示中の弾がひとつも無い場合
            if (this.parent_enemy.flag != 1) // 敵が倒されたら
            {
                this.flag = 0; // 終了
                this.parent_enemy.flag = 0; // 処理不明
            }
        }
        
        /// <summary>
        /// 弾幕と親となる敵が消滅したときの処理をします。
        /// </summary>
        public void death()
        {
            this.flag = 2; // それ以上、弾幕を続けないフラグを立てる
        }

        /// <summary>
        /// 敵ショットと自キャラクタとの処理をします。
        /// </summary>
        /// <returns>自キャラクタが死んだ場合は true</returns>
        public bool enemyshot_and_ch()
        {
            return this.bullets.enemyshot_and_ch();
        }
    }
}
