﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 子ども群を管理するクラスです。
    /// </summary>
    class ChildGroup
    {
        /// <summary>
        /// 子ども群。
        /// </summary>
        private Child[] children = new Child[Setting.CHILD_MAX];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public ChildGroup()
        {
            for (int i = 0; i < this.children.Length; i++)
            {
                this.children[i] = new Child();
            }
        }

        /// <summary>
        /// 子どもにアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>子ども</returns>
        public Child this[int index]
        {
            get
            {
                return this.children[index];
            }
        }

        /// <summary>
        /// 子どもの数を返します。
        /// </summary>
        /// <returns>子どもの数</returns>
        public int length()
        {
            return this.children.Length;
        }
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.children.Length; i++)
                this.children[i].initialize();
        }

        /// <summary>
        /// 計算します。
        /// </summary>
        public void calc_child()
        {
            for (int i = 0; i < this.children.Length; i++)
            {
                Child child = this.children[i];
                if (child.flag > 0)
                    child.calc_child();
            }
        }

        /// <summary>
        /// 空いている子どもを探します。
        /// </summary>
        /// <returns>子ども。空きが無い場合は null</returns>
        public Child search_child()
        {
            for (int i = 0; i < this.children.Length; i++)
            {
                Child child = this.children[i];
                if (child.flag == 0)
                    return child;
            }

            return null;
        }
    }
}
